/*!\file
 * \brief        .
 *
 * Copyright(C)   , 2017
 * Copyright(C)     
 *   .
 *
 *  ,    ,    
 *  ,       
 *  " ".     " " 
 *  - , " "    , 
 *   ,      , 
 *         . 
 *  " "       
 *  ,       ,  , 
 *      ,  - . 
 *  " "         
 *   .  " "     
 *   ,        
 *     .
 *
 *   , ,      
 *       " "  
 *       ,  , 
 *         
 * " ".         .
 */

#include "memory.h"
#include "chip_5023BC016.h"
#include "string.h"
#include "test_memory.h"




/*****************************************************************************
 *  
 ****************************************************************************/



/*     */
void MEMTest_Write(uint32_t Addr, TEST_RADIX Radix, uint32_t Word)
{
    switch(Radix)
    {
    case 4:
        MEM_Write32( Addr, Word);
    break;
    case 2:
        MEM_Write16( Addr, (uint16_t)Word);
    break;
    
    case 1:
        MEM_Write8( Addr, (uint8_t)Word);
    break;              
 }
}

/*     */
uint32_t MEMTest_Read(uint32_t Addr, TEST_RADIX Radix)
{
    switch(Radix)
    {
        case 4:
            return MEM_Read32(Addr);
        case 2:
            return MEM_Read16(Addr);
        case 1:
            return MEM_Read8(Addr);
    }
    return 0;  
}

/*     */
void MEMTest_ZeroBlock(uint32_t Addr, uint32_t Size)
{
      
    for( uint32_t StartAddr = Addr;  StartAddr < Size/sizeof(uint32_t); StartAddr++)
    {
        MEM_Write32( StartAddr, 0);
    }
}

/*   ""*/
bool MEMTEST_Counter(PTEST_MEM pMEMTest)
{
    TEST_RADIX    Radix   = pMEMTest->param.radix;
    uint32_t      Addr    = pMEMTest->block.Addr;
    uint32_t      Size    = pMEMTest->block.Size;
  
    uint32_t Mask = 0;
  
    for(uint8_t i = 0; i < Radix; i++)
    {
        Mask|= 0xFF<<(i*8); 
    }
  
    uint32_t Count = 0;
    uint32_t Errors = 0;
  
    for(uint32_t StartAddr = Addr;  StartAddr < Size/Radix; StartAddr++) 
    {
        MEMTest_Write(StartAddr, Radix, Count);
        Count+=1;
    }
  
    Count = 0;
  
    for(uint32_t StartAddr = Addr;  StartAddr < Size/Radix; StartAddr++) 
    {
        uint32_t r = MEMTest_Read(StartAddr, Radix);
    
    if((Count&Mask)!=r) 
    {
        Errors+=1; 
    }   
    Count+=1;         
    }
    pMEMTest->errors.Errors = Errors;   
    return (Errors)?false:true;
  
}

/*   ""*/
bool MEMTEST_Gallop(PTEST_MEM pMEMTest)
{           
    TEST_RADIX    Radix   = pMEMTest->param.radix;
    uint32_t      Addr    = pMEMTest->block.Addr;
    uint32_t      Size    = pMEMTest->block.Size;
    uint32_t Mask = 0;
    uint32_t Pattern0 = 0;
    uint32_t PatternF = 0;
    for(uint8_t i = 0; i < Radix; i++)
    {
        Mask|= 0xFF<<(i*8);
    }
    PatternF = Mask;
    uint32_t Errors = 0; 
    MEMTest_ZeroBlock( Addr, Size);
  
    for( uint32_t Addr_w = Addr;  Addr_w < Size/Radix; Addr_w++)
    {
        MEMTest_Write( Addr_w, Radix, PatternF);   
        for( uint32_t Addr_r = Addr;  Addr_r < Size/Radix; Addr_r++)
        {
            if(Addr_w!=Addr_r)
            {
                if(MEMTest_Read(Addr_r, Radix)!=Pattern0)
                {
                    Errors+=1;
                }       
            }
        }
        if(!Errors)
        {
            MEMTest_Write( Addr_w, Radix, Pattern0);
        } 
        else 
        {
            MEMTest_ZeroBlock( Addr, Size);
        }    
    }
  
    pMEMTest->errors.Errors = Errors;    
    return (Errors)?false:true;
}

/*   " "*/
bool MEMTEST_Marsch(PTEST_MEM pMEMTest)
{
  
    TEST_RADIX    Radix   = pMEMTest->param.radix;
    uint32_t      Addr    = pMEMTest->block.Addr;
    uint32_t      Size    = pMEMTest->block.Size;
    uint32_t Mask = 0;
    uint32_t Pattern0 = 0;
    uint32_t PatternF = 0;
    for(uint8_t i = 0; i < Radix; i++)
    {
        Mask|= 0xFF<<(i*8);    
    }
    
    PatternF = Mask;
    uint32_t Errors = 0;
    MEMTest_ZeroBlock( Addr, Size);
    for( uint32_t StartAddr = Addr;  StartAddr < Size/Radix; StartAddr++)
    {
        if(MEMTest_Read(StartAddr, Radix)!=Pattern0)
        {
            Errors+=1;
        }
        MEMTest_Write( StartAddr, Radix, PatternF);     
    }
    for( uint32_t StartAddr = Addr;  StartAddr < Size/Radix; StartAddr++) 
    {   
        if(MEMTest_Read(StartAddr, Radix)!=PatternF)
        {
            Errors+=1;
        }
    }   
    
    pMEMTest->errors.Errors = Errors;
    return (Errors)?false:true;    
}

/*****************************************************************************
 *  
 ****************************************************************************/

/**
 * @brief   
 * @param Addr : 
 * @return   
 */
bool MEMTEST_generic(PTEST_MEM pMEMTest)
{
  
    switch(pMEMTest->param.type)
    {
        case TEST_GALOPPING:
            return MEMTEST_Gallop(pMEMTest);
        case TEST_COUNTER:
            return MEMTEST_Counter(pMEMTest);      
        case TEST_MARCHAL:
            return MEMTEST_Marsch(pMEMTest);
            break;
        case TEST_CHESS:
        default:      
            return true;
            break;    
    }
}

