/*!\file
 * \brief         UART.
 *
 * Copyright(C)   , 2017
 * Copyright(C)     
 *   .
 *
 *  ,    ,    
 *  ,       
 *  " ".     " " 
 *  - , " "    , 
 *   ,      , 
 *         . 
 *  " "       
 *  ,       ,  , 
 *      ,  - . 
 *  " "         
 *   .  " "     
 *   ,        
 *     .
 *
 *   , ,      
 *       " "  
 *       ,  , 
 *         
 * " ".         .
 */
#include "uart_test.h"

/*     UART */
#define UART_RB_SIZE 256                                                        //     
const uint32_t sys_freq = 80000000;                                             //  
#pragma data_alignment = 0x400
#pragma location=".noinit"
DMA_CTRL_T DMA_CTRL;                                                            
DMA_CTRL_T DMA_UART_CTRL;
static uint8_t dma_tx_done;
char dbg_buffer[256];                                                           //   
static RINGBUFF_T txring, rxring;
static uint8_t  rxbuff[UART_RB_SIZE],                                           ///<  RX
                txbuff[UART_RB_SIZE];                                           ///<  TX


#define EXT
/**
 * @brief    DMA UART2
 */
void DMA_DONE_UART_2_IRQ(void)
{       
    dma_tx_done = 1;
    UART2->DMACR &= (~UART_DMACR_TXDMAE);                                       //   DMA   
    UART2->DMACR &=(~UART_DMACR_RXDMAE);                                        //   DMA  
    CMN_REG->DMA_INTR_FLAGS &=(~(1<<6));
    CMN_REG->DMA_INTR_FLAGS  |=(1<<6);                                          //   DMA  UART2 TX
}

/**
 * @brief    UART2
 */
void UART_2_IRQ(void)
{
    UART_IRQRBHandler(UART2, &rxring, &txring);                                 //   UART  /    
}       

/**
 * @brief  GPIO  UART2
 */
void gpioInit_UART2()
{
    GPIO_E->ALTFUNCSET = (1 << 2);                                              //uart2 tx
    GPIO_E->ALTFUNCSET = (1 << 1);                                              //uart2 rx
}

/**
 * @brief    UART
 */
uint8_t UART_Test(void)
{
    uint8_t test_rx_buffer[UART_RB_SIZE];
    uint8_t test_tx_buffer[UART_RB_SIZE];
    uint32_t baudrates[12] ={BAUDRATE_921600,                                   //    COM 
                                  BAUDRATE_460800,
                                  BAUDRATE_230400,
                                  BAUDRATE_115200,
                                  BAUDRATE_57600, 
                                  BAUDRATE_38400, 
                                  BAUDRATE_19200, 
                                  BAUDRATE_14400, 
                                  BAUDRATE_9600,  
                                  BAUDRATE_2400,  
                                  BAUDRATE_1200,  
                                  BAUDRATE_110};  

    uint8_t k;  
    volatile uint16_t i,j;    
    UART_Init(UART2);                                                           //  UART
    RingBuffer_Init(&rxring, rxbuff, 1, UART_RB_SIZE);                          //   
    RingBuffer_Init(&txring, txbuff, 1, UART_RB_SIZE);                          //   
    UART_SetBaud(UART2, BAUDRATE_921600,sys_freq);                              //            
    UART_Config(UART2, (UART_LCR_H_PDIS |                                       //    ,      
                        UART_LCR_H_EPS_ODD |
                        UART_LCR_H_STP1 |
                        UART_LCR_H_FEN | 
                        UART_LCR_H_WLEN8 | 
                        UART_LCR_H_SPS_DIS));    
    UART_Enable(UART2);                                                         //   UART
    UART_TXEnable(UART2);                                                       //   
    UART_RXEnable(UART2);                                                       //   
    UART_SetupFIFOS(UART2, UART_IFLS_1DIV8, UART_IFLS_1DIV8);                   //       /    
    for (i = 0; i < (UART_RB_SIZE); i++) 
    {
        test_tx_buffer[i] = i;
        test_rx_buffer[i] = 0xAA;
    }   
    gpioInit_UART2();                                                           //  GPIO  UART2
    PRINT(" UART");
    PRINT("    :")
    for (k = 0; k <= 12; k+=3) 
    {
      if (k<10)
        PRINT(" %i/, %i/, %i/, ", baudrates[k], baudrates[k+1], baudrates[k+2]);
    } 
        DBG_TX_Flush();                                                         //   ,    ,   UART      

/*****************************************************************************
 *  
 ****************************************************************************/
          
   for (k = 0; k <= 11; k++) 
    {
         UART_SetBaud(UART2, baudrates[k],sys_freq);                            //            
         i = 0;
         j = 0;
         while (j < (UART_RB_SIZE))  
         { 
              while ((UART_GetStatus(UART2, UART_FR_RXFE)) &&                   //  UART , SET or RESET
                     !(UART_GetStatus(UART2, UART_FR_TXFF)) && 
                       (i < (UART_RB_SIZE))) 
              {
                  UART_SendByte(UART2, test_tx_buffer[i]);                      //     UART'
                  i++;
              }
              while (!(UART_GetStatus(UART2, UART_FR_RXFE)))                    //  UART , SET or RESET
              {
                  test_rx_buffer[j] = UART_ReadByte(UART2);                     //     UART'
                  j++;
              }
         }
         if (memcmp(test_tx_buffer, test_rx_buffer, (UART_RB_SIZE))) 
         {
            PRINT("      %i ",
                  baudrates[k]);
            DBG_TX_Flush();                                                     //   ,    ,   UART
           // return false;
         }
         memset(test_rx_buffer, 0, UART_RB_SIZE);
    }
    UART_DeInit(UART2);                                                         //  UART
    CLK_UART_PAUSE;  
    PRINT("    ");
    DBG_TX_Flush();                                                             //   ,    ,   UART    
       
/*****************************************************************************
 *  c 
 ****************************************************************************/        
        
    PRINT("    UART   c ");
    DBG_TX_Flush();                                                             //   ,    ,   UART     
    int result=0;
    for (k = 0; k < 11; k++) 
    {
        UART_Init(UART2);                                                       //  UART
        CLK_UART_PAUSE;       
        UART_SetBaud(UART2, baudrates[k],sys_freq);                             //            
        UART_Config(UART2,(UART_LCR_H_PDIS |                                    //    ,      
                            UART_LCR_H_EPS_ODD |
                              UART_LCR_H_STP2 | 
                                UART_LCR_H_FEN |
                                  UART_LCR_H_WLEN8 | 
                                    UART_LCR_H_SPS_DIS));
        UART_Enable(UART2);                                                     //   UART
        UART_TXEnable(UART2);                                                   //   
        UART_RXEnable(UART2);                                                   //   
        UART_SetupFIFOS(UART2, UART_IFLS_1DIV8, UART_IFLS_1DIV8);               //       /
        CLK_UART_PAUSE;
        UART_IntEnable(UART2, UART_IMSC_RXIM);                                  //    UART'
        UART_IntEnable(UART2, UART_IMSC_TXIM);                                  //    UART'
        NVIC_EnableIRQ(UART_2,3);                                               //  
        CLK_UART_PAUSE;
        for (i = 0; i < (UART_RB_SIZE); i++) 
        {
            test_tx_buffer[i] = i;
            test_rx_buffer[i] = 0xAA;
        }          
        UART_SendRB(UART2, &txring, test_tx_buffer, (UART_RB_SIZE));            //           
        CLK_UART_PAUSE;
        int l = 0;          
        while ((RingBuffer_GetCount(&rxring) < 256) && (l < 20000000))
        {
            if (!(UART_GetStatus(UART2, UART_FR_RXFE))) 
            {
                uint8_t ch = UART_ReadByte(UART2);
                RingBuffer_Insert(&rxring, &ch);
            }
            l++;
        }
        UART_ReadRB(UART2, &rxring, test_rx_buffer, (UART_RB_SIZE));            //         
        if (memcmp(test_tx_buffer, test_rx_buffer, (UART_RB_SIZE))) 
        {
#ifndef NO_VERBOSE     

            PRINT("       UART   c    %i", baudrates[k]);
            DBG_TX_Flush();                                                     //   ,    ,   UART
#endif            
        result = 1;
        }
        else
        {
            PRINT("  UART   c    %i", baudrates[k]);
            DBG_TX_Flush();                                                     //   ,    ,   UART
        }
        NVIC_DisableIRQ(3);                                                     //   
        UART_IntDisable(UART2, UART_IMSC_RXIM);                                 //    UART'
        UART_IntDisable(UART2, UART_IMSC_TXIM);                                 //    UART'    
        UART_DeInit(UART2);                                                     //  UART
        CLK_UART_PAUSE;
    }       
    if (result) 
    {
        PRINT("    UART   c ");
    }  
    else
    {
        PRINT(" UART   c   ! ");
    }
    DBG_TX_Flush();                                                             //   ,    ,   UART
    UART_DeInit(UART2);                                                         //  UART
    
/*****************************************************************************
 *  c DMA (    )
 ****************************************************************************/        
    int result_dma =0;
#ifndef EXT   

    PRINT("    UART   c DMA");
    DBG_TX_Flush();                                                             //   ,    ,   UART
    for (k = 0; k < 11; k++) 
    {
        UART_Init(UART2);                                                       //  UART
        CLK_UART_PAUSE;
        UART_SetBaud(UART2, baudrates[k],sys_freq);                             //            
        UART_Config(UART2, (UART_LCR_H_PDIS |                                   //    ,      
                            UART_LCR_H_EPS_ODD | 
                              UART_LCR_H_STP1 |
                                UART_LCR_H_FEN | 
                                  UART_LCR_H_WLEN8 | 
                                    UART_LCR_H_SPS_DIS));
        UART_Enable(UART2);                                                     //   UART                                               
        UART_TXEnable(UART2);                                                   //   
        UART_RXEnable(UART2);                                                   //   
        UART_SetupFIFOS(UART2, UART_IFLS_1DIV8, UART_IFLS_1DIV8);               //       /
        CLK_UART_PAUSE;     
        UART_IntEnable(UART2, UART_IMSC_RXIM);                                  //    UART'
        UART_IntEnable(UART2, UART_IMSC_TXIM);                                  //    UART'     
        CLK_UART_PAUSE;          
        int p= 0;
        for (p = 0; p < (UART_RB_SIZE); p++)
        {
            test_tx_buffer[p] = p;
            test_rx_buffer[p] = 0xAA;
        }  
        //  DMA
        DMA->CFG = 1; 
        DMA->CTRL_BASE_PTR =  (uint32_t) &DMA_UART_CTRL;
        DMA->CHNL_ENABLE_SET = (1<<6);      
        DMA_UART_CTRL.prim_chan[6].src_data_end_ptr = (uint32_t)&test_tx_buffer[UART_RB_SIZE - 1];
        DMA_UART_CTRL.prim_chan[6].dst_data_end_ptr = (uint32_t)UART2_BASE;
        DMA_UART_CTRL.prim_chan[6].ctrl = DMA_CCTRL_BASIC | DMA_NEXT_USEBURST_DIS | \
                                       DMA_N_MINUS_1(UART_RB_SIZE - 1) | DMA_R_POWER(4) | \
                                       DMA_SRC_PROT_CACHE_DIS | DMA_SRC_PROT_BUFFER_DIS | DMA_SRC_PROT_PRIVILEGE_DIS | \
                                       DMA_DST_PROT_CACHE_DIS | DMA_DST_PROT_BUFFER_DIS | DMA_DST_PROT_PRIVILEGE_DIS | \
                                       DMA_SRC_DATA_SIZE_BYTE | DMA_SRC_ADDR_INC_BYTE | \
                                       DMA_DST_DATA_SIZE_BYTE | DMA_DST_ADDR_INC_NULL;  
        CMN_REG->DMA_INTR_FLAGS &=(~(1<<6)); 
        dma_tx_done = 0;
        UART2->DMACR |= UART_DMACR_TXDMAE;                                      //   DMA  
        while (!(UART_GetStatus(UART2, UART_FR_TXFE))) {}                       //    UART 
        int l = 0;          
        while ((RingBuffer_GetCount(&rxring) < 256) && (l < 20000000))          //   ,    .
        {

            if (!(UART_GetStatus(UART2, UART_FR_RXFE))) 
            {
                uint8_t ch = UART_ReadByte(UART2);                              
                RingBuffer_Insert(&rxring, &ch);
            }
            
            l++;
        }
        UART_ReadRB(UART2, &rxring, test_rx_buffer, (UART_RB_SIZE));            //       
        if (memcmp(test_tx_buffer, test_rx_buffer, UART_RB_SIZE)) 
        {
#ifndef NO_VERBOSE             
            PRINT("       UART   c DMA    %i", baudrates[k]);
            DBG_TX_Flush();                                                     //   ,    ,   UART
#endif            
            result_dma = 1;
        }
        else
        {
                
            PRINT("  UART   c DMA    %i", baudrates[k]);
            DBG_TX_Flush();                                                     //   ,    ,   UART
        }
        memset(test_rx_buffer, 0, UART_RB_SIZE);
        DMA->CFG = 0; 
        UART_IntDisable(UART2, UART_IMSC_RXIM);                                 //    UART'
        UART_DeInit(UART2);                                                     //  UART
        CLK_UART_PAUSE;
    }    
    if (result_dma)                             
    {
        PRINT("    UART   c DMA");
    }  
    DBG_TX_Flush();                                                             //   ,    ,   UART
#endif        
    if ((!result_dma)&&(!result))
    {
        PRINT(" UART  ");
    }
    else
    {
        PRINT("    UART");
    }  
    DBG_TX_Flush();                                                             //   ,    ,   UART
    return result_dma;         
}
