
/*!\file
 * \brief         Manchester.
 *
 * Copyright(C)   , 2017
 * Copyright(C)     
 *   .
 *
 *  ,    ,    
 *  ,       
 *  " ".     " " 
 *  - , " "    , 
 *   ,      , 
 *         . 
 *  " "       
 *  ,       ,  , 
 *      ,  - . 
 *  " "         
 *   .  " "     
 *   ,        
 *     .
 *
 *   , ,      
 *       " "  
 *       ,  , 
 *         
 * " ".         .
 */


#include "manchester_test.h"   

const uint32_t sys_freq = 80000000;                                             ///<  
char dbg_buffer[256];                                                           ///<  

#define SYS_FREQ 80                                                             ///<         MKPD                                                  
#define INTERNAL_TEST                                                           //  
#define MKPD_MAX_WORD_IN_PACKET         32                                      /**<        */ 
#define MKPD_EP_SUBADDR_COUNT           32                                      /**<      */                                  

unsigned int EP1_addr;
unsigned int EP2_addr;
unsigned int reply_error = 0;

int com_error = 0;
int data_error = 0; 
int monitor_error = 0;

uint8_t BC_RX_Buffer[MKPD_MAX_WORD_IN_PACKET * 2];                              ///<       
   
uint8_t BC_TX_Buffer[MKPD_MAX_WORD_IN_PACKET * 2];                              ///<       
 
Command_Word_T ep_command1;                                                     ///<   ,    1
   
MKPD_T * mkpd[4] = {MKPD1, MKPD2,MKPD3, MKPD4};

#ifndef INTERNAL_TEST

#if (FREQ != SYS_FREQ)
#error Manchester frequency has not correct value (speed will not be equal to 2 Mbit/s)
#endif

#endif

volatile unsigned int irq_counter[4] = {0,0,0,0};

void Manchester1_IRQ()
{
    irq_counter[0]++;
}

void Manchester2_IRQ()
{
    irq_counter[1]++;
}

void Manchester3_IRQ()
{
    irq_counter[2]++;
}

void Manchester4_IRQ()
{
    irq_counter[3]++;            
}

//   ,   
void CheckCommandWord(Command_Word_T command, uint32_t ep_addr, uint32_t tx_rx, uint32_t ep_subaddr, uint32_t word_count)
{ 
    if ((command.bit.EP_addr                     != ep_addr) ||
            (command.bit.Tx_rx_mark                  != tx_rx) || 
            (command.bit.Command_word_mark           != 1) ||
            (command.bit.Subaddr__Manage_mode        != ep_subaddr) ||
            (command.bit.Data_size__Command_code     != (word_count % MKPD_MAX_WORD_IN_PACKET)))
                com_error++;
}

unsigned int com_error1 = 0;
unsigned int reply_error1 = 0;


/**
 * @brief	   ()
 */ 
typedef struct 
{
    volatile unsigned short  FEP1:1;                                            /**<  " " */
    volatile unsigned short  MAN:1;                                             /**<  "  " */
    volatile unsigned short  FEP0:1;                                            /**<  " " */
    volatile unsigned short  BUSY:1;                                            /**<  " " */
    volatile unsigned short  Group_rx:1;                                        /**<  "  " */
    volatile unsigned short  RES:3;                                             /**<   */
    volatile unsigned short  SREQ:1;                                            /**<  "  " */
    volatile unsigned short  Reply_tx:1;                                        /**<  "  " */
    volatile unsigned short  Data_err:1;                                        /**<  "  " */
    volatile unsigned short  EP_addr:5;                                         /**<    */
}Reply_Word_Bits;

/**
 * @brief	        
 */ 
typedef union Reply_Word
{
volatile unsigned short               all;                                      /**<      */
struct Reply_Word_Bits          bit;                                            /**<      */
} Reply_Word_T;

//  ,   .   16 
uint32_t Word_count = MKPD_MAX_WORD_IN_PACKET;
//  ,      
Reply_Word_T reply_word;

#define INPORT_MUX              ((INPORT_MUX_t*) 0xA0FF0000)
#define OUTPORT_MUX             ((OUTPORT_MUX_t*)  0xA0FF1000)
/**
 * @brief	  MUX
 */ 
struct INPORT_MUX_BITS
{
    unsigned int NUM_FUNC   :8;                                                 /**< bit 0 - 7 */                        
    unsigned int rsrv1      :8;                                                 /**< bit 8 - 15 */                     
    unsigned int LOOPBACK   :1;                                                 /**< bit 16 */                        
    unsigned int rsrv2      :14;                                                /**< bit 17 - 30 */
    unsigned int ENA        :1;                                                 /**< bit 31 */ 
};

/**
 * @brief	  MUX
 */
union INPORT_MUX_REG
{
    volatile unsigned int           all;
    struct INPORT_MUX_BITS          bit;
};

/**
 * @brief	 INPORT_MUX
 */
typedef struct
{
    volatile unsigned int   MCU_PORT[144];
}INPORT_MUX_t;

/**
 * @brief	  MUX
 */
struct OUTPORT_MUX_BITS
{
    unsigned int NUM_PORT   :8;                                                 /**< bit 0 - 7 */                        
    unsigned int rsrv1      :23;                                                /**< bit 8 - 30 */                     
    unsigned int ENA        :1;                                                 /**< bit 31 */ 
};

/**
 * @brief	  MUX
 */
union OUTPORT_MUX_REG
{
    volatile unsigned int           all;
    struct OUTPORT_MUX_BITS          bit;
};

/**
 * @brief	 OUTPORT_MUX
 */
typedef struct
{
    volatile unsigned int   FPGA_PORT[250];  
} OUTPORT_MUX_t;

void MKPD_Clock_Init()
{
    PWR_CLK_Enable(MKPD1_CTRL_NUM); 
    CLK_INIT_PAUSE;
    PWR_RST_Disable(MKPD1_CTRL_NUM);
    
    PWR_CLK_Enable(MKPD2_CTRL_NUM);
    CLK_INIT_PAUSE;
    PWR_RST_Disable(MKPD2_CTRL_NUM);
    
    PWR_CLK_Enable(MKPD3_CTRL_NUM); 
    CLK_INIT_PAUSE;
    PWR_RST_Disable(MKPD3_CTRL_NUM);
    
    PWR_CLK_Enable(MKPD4_CTRL_NUM);
    CLK_INIT_PAUSE;
    PWR_RST_Disable(MKPD4_CTRL_NUM);
    
}

void MKPD_configure_time()
{
    //     
    for (uint8_t index = 0; index < 4; index++)
    {
          
        switch (index)
        {
            case 0:
                MKPD_Set_Interpacket_Time(mkpd[index], 0xF);
                break;
            case 1:
                MKPD_Set_Interpacket_Time(mkpd[index], 0xF);
                break;
            case 2:
                MKPD_Set_Interpacket_Time(mkpd[index], 0xF);
                break;
            case 3:
                MKPD_Set_Interpacket_Time(mkpd[index], 0xF);
                break;
        }       
          
        MKPD_Set_Wait_Time(mkpd[index], 38);                                    // >= 21 mks
        MKPD_Set_Edge_Time(mkpd[index], SYS_FREQ/5);                            // > 0.2 mks  
        MKPD_Set_System_Clock(mkpd[index], SYS_FREQ);
        MKPD_Set_InvTX(mkpd[index], 1);
        MKPD_Set_IncWith(mkpd[index], 0);
        MKPD_Set_DecoderDisable(mkpd[index], 0x0);                              //all enabled
                
        MKPD_Set_NoActivity_Time(mkpd[index], SYS_FREQ * 13 / 10);              // > 1.3 mks
        MKPD_Set_BCWaitReply_Time(mkpd[index], 36);                             // > 35 mks
        MKPD_Set_Clk_500NS(mkpd[index], SYS_FREQ / 2);              
        MKPD_Set_Clk_1250NS(mkpd[index], SYS_FREQ * 125 / 100);
        MKPD_Set_Clk_1500NS(mkpd[index], SYS_FREQ * 150 / 100);
        MKPD_Set_Clk_1750NS(mkpd[index], SYS_FREQ * 175 / 100);
        MKPD_Set_Clk_2250NS(mkpd[index], SYS_FREQ * 225 / 100);          
    }
}

//   
void CheckReplyWord(uint32_t ep_addr, uint32_t group_rx)
{
    if ((reply_word.bit.EP_addr     != ep_addr) ||
       (reply_word.bit.Data_err    != RESET) ||
       (reply_word.bit.Reply_tx    != RESET) ||  
       (reply_word.bit.SREQ        != RESET) ||
       (reply_word.bit.Group_rx    != group_rx) ||
       (reply_word.bit.BUSY        != RESET) ||  
       (reply_word.bit.FEP0        != RESET) ||
       (reply_word.bit.MAN         != RESET) ||
       (reply_word.bit.FEP1        != RESET)) 
       reply_error++;
}

//      
uint8_t ErrorReport()
{
    uint8_t errors = 0;
     
    if (data_error > 0 )
    {
        PRINT("Error in subtest\n");               
     
        PRINT("\tData word errors: %d\n", data_error);
                            DBG_TX_Flush();                                     //   ,    ,   UART

        errors = 1;
    }
    else 
    {       
         PRINT("\t\t\tSubtest complete - OK\n");             
    }
        
    com_error = 0;
    reply_error = 0;
    data_error = 0;      
    monitor_error = 0;
                
    return errors;
}


void Update_EP_addrs()
{
  
    EP1_addr = (rand() % 30) + 1;
        EP2_addr = (rand() % 30) + 1;
        while (EP2_addr == EP1_addr)
                EP2_addr = (rand() % 30) + 1;
         
}

void FPGA_MUX_Disable_all()
{
    //.: ..    ,        
    ////   ( extmem  uart1)
    for (unsigned int port = 55; port < 250; port++)
    {
        if (port != 67 &&
            port != 117 &&
            port != 118 &&
            port != 119
            )
        {
            if (port < 144)
                INPORT_MUX->MCU_PORT[port] = 0;                    
            OUTPORT_MUX->FPGA_PORT[port] = 0;                       
        }
    }

    INPORT_MUX->MCU_PORT[64] = 64 | (1 << 31);
    OUTPORT_MUX->FPGA_PORT[63] = 63 | (1 << 31);
}
void MKPD_init()
{
    MKPD_Clock_Init();                                                          //   

    NVIC_EnableIRQ(MKPD1_NUM,20);                                               //  
    NVIC_EnableIRQ(MKPD2_NUM,21);
    NVIC_EnableIRQ(MKPD3_NUM,22);
    NVIC_EnableIRQ(MKPD4_NUM,23);    
    
        //     
        for (uint8_t index = 0; index < 4; index++)
        {

          
                switch (index)
                {
                case 0:
                        MKPD_Set_Interpacket_Time(mkpd[index], 0xF);
                        break;
                case 1:
                        MKPD_Set_Interpacket_Time(mkpd[index], 0xF);
                        break;
                case 2:
                        MKPD_Set_Interpacket_Time(mkpd[index], 0xF);
                        break;
                case 3:
                        MKPD_Set_Interpacket_Time(mkpd[index], 0xF);
                        break;
                }       
          
                MKPD_Set_Wait_Time(mkpd[index], 38);                            // >= 21 mks
                MKPD_Set_Edge_Time(mkpd[index], SYS_FREQ/5);                    // > 0.2 mks  
                MKPD_Set_System_Clock(mkpd[index], SYS_FREQ);
                MKPD_Set_InvTX(mkpd[index], 1);
                MKPD_Set_IncWith(mkpd[index], 0);
                MKPD_Set_DecoderDisable(mkpd[index], 0x0);                      //all enabled
                
                //MKPD_Set_NoActivity_Time(mkpd[index], 185);      // > 1.3 mks
                MKPD_Set_NoActivity_Time(mkpd[index], SYS_FREQ * 13 / 10);      // > 1.3 mks
                MKPD_Set_BCWaitReply_Time(mkpd[index], 36);                     // > 35 mks
                MKPD_Set_Clk_500NS(mkpd[index], SYS_FREQ / 2);              
                MKPD_Set_Clk_1250NS(mkpd[index], SYS_FREQ * 125 / 100);
                MKPD_Set_Clk_1500NS(mkpd[index], SYS_FREQ * 150 / 100);
                MKPD_Set_Clk_1750NS(mkpd[index], SYS_FREQ * 175 / 100);
                MKPD_Set_Clk_2250NS(mkpd[index], SYS_FREQ * 225 / 100);          
        }
}

//     MKPD:     
uint8_t TestTwoBlocks2(MKPD_T * ManBC, MKPD_T * ManEP, MKPD_SETTING0_CHANNEL_T Channel)
{  

        Update_EP_addrs();
  
        //    1
        uint8_t* EP1_Buffer = malloc(MKPD_MAX_WORD_IN_PACKET * MKPD_EP_SUBADDR_COUNT * 2 * sizeof(uint8_t));

        //    2
        uint8_t* EP2_Buffer = malloc(MKPD_MAX_WORD_IN_PACKET * MKPD_EP_SUBADDR_COUNT * 2 * sizeof(uint8_t));

        com_error = 0;
        reply_error = 0;
        data_error = 0; 
        monitor_error = 0;
  
        //  
        MKPD_ConfigBusController(ManBC, (uint32_t)BC_RX_Buffer, (uint32_t)BC_TX_Buffer, Channel);       
        
        //  
        MKPD_ConfigEndpoint(ManEP, (uint32_t)EP1_Buffer, EP1_addr);
                              
        //   
        MKPD_Set_Endpoint_Attributes(ManEP, 0);
        
        for (int i=0; i < MKPD_MAX_WORD_IN_PACKET * MKPD_EP_SUBADDR_COUNT * 2; i++)
	{
		EP1_Buffer[i] = i;
	}
    
	for (int i=0; i < MKPD_EP_SUBADDR_COUNT * 2; i++)
	{
		BC_TX_Buffer[i] = 0;
		BC_RX_Buffer[i] = 0xFF;
	}
          
        unsigned char stop_test = 0;
        for (Word_count = 1; Word_count < 33; Word_count++)
        {
                if (stop_test)
                        break;
                //uint32_t EP_subaddr = 0;
                for (uint32_t EP_subaddr = 1; EP_subaddr < MKPD_EP_SUBADDR_COUNT - 1; EP_subaddr++)
                {
                        if (MKPD_StartMessage(ManBC, 
                                              TX_ONE, 
                                              2, 
                                              EP1_addr, 
                                              EP_subaddr, 
                                              0, 
                                              0, 
                                              Word_count,
                                              10000, 
                                              1) != 0)
                        {
                                PRINT("Error: BC busy timeout\n");
                                DBG_TX_Flush();                                 //   ,    ,   UART

                                stop_test = 1;
                                break;
                        }
                        
                        //   ,   
                        ep_command1.all = MKPD_Get_Endpoint_Command_Word(ManEP); 
                        CheckCommandWord(ep_command1, EP1_addr, 1, EP_subaddr, Word_count);
                                        
                        //   ,   
                        reply_word.all = MKPD_BC_Reply_Word(ManBC, TX_ONE);
                        CheckReplyWord(EP1_addr, 0);
                                        
                        //                         
                        for (uint32_t i = 0; i< Word_count*2; i++)
                                if (BC_RX_Buffer[i] != EP1_Buffer[i+(EP_subaddr-1)*64])
                                        data_error++;                
                }
        }
                                  
        free(EP1_Buffer);
        free(EP2_Buffer);
        MKPD_ConfigEndpoint(ManEP, (uint32_t)EP1_Buffer, 0);
        
        return ErrorReport(); 
}

//   manchester      
#warning       -   mode == 0,    mode == 1.      RAM_DATA
void Init_ports_Manchester(unsigned int mode)
{
    FPGA_MUX_Disable_all(); 
  
    PWR_CLK_Disable(MKPD1_CTRL_NUM); 
    CLK_INIT_PAUSE;
    PWR_RST_Enable(MKPD1_CTRL_NUM);
    PWR_CLK_Disable(MKPD2_CTRL_NUM);
    CLK_INIT_PAUSE;
    PWR_RST_Enable(MKPD2_CTRL_NUM);
    PWR_CLK_Disable(MKPD3_CTRL_NUM); 
    CLK_INIT_PAUSE;
    PWR_RST_Enable(MKPD3_CTRL_NUM);
    PWR_CLK_Disable(MKPD4_CTRL_NUM);
    CLK_INIT_PAUSE;
    PWR_RST_Enable(MKPD4_CTRL_NUM);

        if (mode == 0)
        {     
                //config altfunc                
          CMN_REG->ALT_FUNCTION_CTRL[4] &= 0x000000FF;                          //E4-E15 - func 1
                CMN_REG->ALT_FUNCTION_CTRL[5]   = 0x00000000;
                CMN_REG->ALT_FUNCTION_CTRL[6]  &= 0xFFFFFF00;
                
                //enable altfunc in gpio       
                GPIO_E->ALTFUNCSET = 0xFFF0;                                    //E4-15
                GPIO_F->ALTFUNCSET = 0xFFFF;                                    //F0-15
                GPIO_G->ALTFUNCSET = 0x000F;                                    //G0-3
                
                for (int i = 0; i < 32; i++)
                {
                        INPORT_MUX->MCU_PORT[68 + i] = (68 + i) | (1 << 31);    
                        OUTPORT_MUX->FPGA_PORT[68 + i] = (68 + i) | (1 << 31); 
                }
        }
                        
    MKPD_Clock_Init();                                                          //   

}

#define TEST_CHANNEL_A
#define TEST_CHANNEL_B

int MKPD_Test()
{
            PRINT("\n********** MKPD test start **********\n");
            DBG_TX_Flush();                                                     //   ,    ,   UART
                                

        Init_ports_Manchester(0);
        
        MKPD_init();    
          
        uint32_t errors = 0;
        
        Update_EP_addrs();   

        PRINT("\t\tTest blocks in pairs\n");
        DBG_TX_Flush();                                                         //   ,    ,   UART

        for (uint8_t BC_index = 0; BC_index < 4; BC_index++)
                for (uint8_t EP_index = 0; EP_index < 4; EP_index++)
                        if (EP_index != BC_index)
                        {
#ifdef TEST_CHANNEL_A
                                PRINT("\t\t\tBC = %d, EP = %d, Channel A\n", BC_index, EP_index);
                                DBG_TX_Flush();                                                     //   ,    ,   UART

                                errors += TestTwoBlocks2(mkpd[BC_index], mkpd[EP_index], MKPD_SETTING0_CHANNEL_A); 
#endif
#ifdef TEST_CHANNEL_B
                                PRINT("\t\t\tBC = %d, EP = %d, Channel B\n", BC_index, EP_index);
                                DBG_TX_Flush();                                                     //   ,    ,   UART

                                errors += TestTwoBlocks2(mkpd[BC_index], mkpd[EP_index], MKPD_SETTING0_CHANNEL_B);
#endif
                        }    
        return 0;
}

