/*!\file
 * \brief        .
 *
 * Copyright(C)   , 2017
 * Copyright(C)     
 *   .
 *
 *  ,    ,    
 *  ,       
 *  " ".     " " 
 *  - , " "    , 
 *   ,      , 
 *         . 
 *  " "       
 *  ,       ,  , 
 *      ,  - . 
 *  " "         
 *   .  " "     
 *   ,        
 *     .
 *
 *   , ,      
 *       " "  
 *       ,  , 
 *         
 * " ".         .
 */

//tests of internal memory

#include "common.h"
#include "chip_5023BC.h"
#include "debug_uart.h"

#define RAM_SIZE        (128*1024)  //256K, EDAC is ON

#define RAM_START       0x20000000
#define RAM_END         (0x20000000+RAM_SIZE - 1)

#ifdef RAM
  #define START_ADR       (RAM_START + PROGRAM_SIZE)  //    ,   
#else
  #ifdef EXT
    #define START_ADR       RAM_START
  #else
    #error program load configuration is undefined 
  #endif
#endif //RAM

#define END_ADR         RAM_END

#define M32

#ifdef M32
  #define INC     4
  #define TYPE    unsigned int
  #define PATTERN_F     0xFFFFFFFF
  #define PATTERN_0     0x00000000
  #define BIT_COUNT     32
#else
  #ifdef M16
    #define INC     2
    #define TYPE    unsigned short
    #define PATTERN_F     0xFFFF
    #define PATTERN_0     0x0000
    #define BIT_COUNT     16
  #else
    #ifdef M8
      #define INC     1
      #define TYPE    unsigned char
      #define PATTERN_F     0xFF
      #define PATTERN_0     0x00
      #define BIT_COUNT     8
    #else
  #error memory read mode is undefined
    #endif //M8
  #endif //M16

#endif //M32


unsigned char ErrorsDetected;

void ReportError(unsigned int code, unsigned int adr, unsigned int val1, unsigned int val2)
{             
        PRINT("                       ");
        PRINT_INT(code);
        PRINT(" ");
        PRINT_INT_HEX(adr);
        PRINT(" ");
        PRINT_INT_HEX(val1);
        PRINT(" ");
        PRINT_INT_HEX(val2);
        PRINT("\n");
                       
        ErrorsDetected++;      
}

void ClearAll()
{  
        for(unsigned int adr=START_ADR; adr<=END_ADR; adr+=INC)
                (*(TYPE*)adr) = PATTERN_0;
}

volatile unsigned int w_adr; //write address
volatile unsigned int r_adr; //read address

#define TEMP    (START_ADR + RAM_SIZE/32 - 1) //END_ADR  

void Galloping_Test()
{        
        ClearAll();
        
        for(w_adr=START_ADR; w_adr<=TEMP; w_adr+=INC)
        {
                (*(TYPE*)w_adr) = PATTERN_F;
                for(int r_adr=START_ADR; r_adr<=TEMP; r_adr+=INC)
                        if(r_adr != w_adr)                                
                        {                                  
                                if(*(TYPE*)r_adr != PATTERN_0)                                
                                        ReportError(0, r_adr, w_adr, *(TYPE*)r_adr);
                                if(*(TYPE*)w_adr != PATTERN_F)                                
                                        ReportError(1, r_adr, w_adr, *(TYPE*)w_adr);
                                
                        }                
                ClearAll();
        }  
}


void Marsch_Test()
{
        for(unsigned int adr=START_ADR; adr<=END_ADR; adr+=INC)
                (*(TYPE*)adr) = PATTERN_0;
        for(unsigned int adr=START_ADR; adr<=END_ADR; adr+=INC)
        {      
                if ( *(TYPE*)adr != PATTERN_0)
                        ReportError(2, adr, PATTERN_0, *(TYPE*)adr);
                (*(TYPE*)adr) = PATTERN_F;
        }                
        for(unsigned int adr=START_ADR; adr<=END_ADR; adr+=INC)
        {      
                if ( *(TYPE*)adr != PATTERN_F)
                        ReportError(3, adr, PATTERN_F, *(TYPE*)adr);
                (*(TYPE*)adr) = PATTERN_0;
        }
        for(unsigned int adr=END_ADR-3; adr>=START_ADR; adr-=INC)
        {      
                if ( *(TYPE*)adr != PATTERN_0)
                        ReportError(4, adr, PATTERN_0, *(TYPE*)adr);
                (*(TYPE*)adr) = PATTERN_F;
        }
        for(unsigned int adr=END_ADR-3; adr>=START_ADR; adr-=INC)
        {      
                if ( *(TYPE*)adr != PATTERN_F)
                        ReportError(5, adr, PATTERN_F, *(TYPE*)adr);
                (*(TYPE*)adr) = PATTERN_0;
        }
        for(unsigned int adr=END_ADR-3; adr>=START_ADR; adr-=INC)      
                if ( *(TYPE*)adr != PATTERN_0)
                        ReportError(6, adr, PATTERN_0, *(TYPE*)adr);   
}

void Run01_Test()
{      
        volatile TYPE value;
        
        //running 1
        value = 1;
        for(int pos = 0; pos < BIT_COUNT; pos++)
        {
                for(unsigned int adr=START_ADR; adr<=END_ADR; adr+=INC)
                        *(TYPE*)adr = value;
          
                for(unsigned int adr=START_ADR; adr<=END_ADR; adr+=INC)
                        if (*(TYPE*)adr != value)
                                ReportError(7, adr, value, *(TYPE*)adr);
                
                value <<= 1;        
        }
        
        //running 0
        value = 1;
        for(int pos = 0; pos < BIT_COUNT; pos++)
        {
                for(unsigned int adr=START_ADR; adr<=END_ADR; adr+=INC)
                        *(TYPE*)adr = ~value;
          
                for(unsigned int adr=START_ADR; adr<=END_ADR; adr+=INC)
                        if ((*(TYPE*)adr) != ((~value) & PATTERN_F))
                                ReportError(8, adr, ((~value) & PATTERN_F), *(TYPE*)adr);                                             
                value <<= 1;        
        }        
}

void Counter_Test()
{
        for(unsigned int adr=START_ADR; adr<=END_ADR; adr+=INC)
                *(TYPE*)adr = (TYPE)adr;
        
        for(unsigned int adr=START_ADR; adr<=END_ADR; adr+=INC)
                if ( *(TYPE*)adr != (TYPE)adr )
                        ReportError(9, adr, (TYPE)adr, *(TYPE*)adr);                    
       
        for(unsigned int adr=START_ADR; adr<=END_ADR; adr+=INC)
                *(TYPE*)adr = (TYPE)~adr;
        
        for(unsigned int adr=START_ADR; adr<=END_ADR; adr+=INC)
                if ( *(TYPE*)adr != (TYPE)~adr ) 
                        ReportError(10, adr, (TYPE)~adr, *(TYPE*)adr); 
}


void Chess_Test()
{
        #define TEST_CONST1     0xAAAAAAAA
        #define TEST_CONST2     0x55555555
        #define MEMORY_EXPOSURE_TIME    1000000
  
        unsigned char mem_test_state = 0;
        unsigned int test_const;
        
        if (mem_test_state++)
                test_const = TEST_CONST1;
        else
                test_const = TEST_CONST2;
        mem_test_state %=2;

        unsigned short cerr;
        unsigned short ferr;
        unsigned int errors_count = 0;
        
        EDAC_DISABLE;
        EDAC_ENABLE;
        
        //unsigned char counter = 0;
        for (unsigned int adr = START_ADR; adr < END_ADR; adr+=4)
        {   
                *(unsigned int*)adr = test_const;          
        }
                
        //pause
        for(unsigned int i=0; i<MEMORY_EXPOSURE_TIME; i++);
        
        //  
        //*(unsigned char*)0x20003214 = 0xFF;
        //*(unsigned char*)0x20001520 = 0xEE;
        //*(unsigned char*)0x20007F51 = 0xDD;
        
        //for (unsigned int adr = START_ADR; adr < END_ADR; adr+=4)
                //if (adr % 120 == 0)
        //                *(unsigned int*)adr = ~test_const;
        
        //counter = 0;
        for (unsigned int adr = START_ADR; adr < END_ADR; adr+=4)
        {
                if (*(unsigned int*)adr != test_const)
                {
                        ReportError(11, adr, test_const, *(unsigned int*)adr);                      
                                                                                                                                                                                                                                            
                        unsigned int val = *(unsigned int*)adr;
                        for (unsigned char pos = 0; pos<32; pos++)
                                if ( ((val >> pos) & 1) != ((test_const >> pos) & 1) )
                                {                                          
                                        errors_count++;                                           
                                } 
                         
                }  
                     
        }
        if (errors_count > 0)
        {
                //for(int i=0; i<errors_count; i++);
        }
        
        for (unsigned int adr = START_ADR; adr < END_ADR; adr+=4)
        {
                *(unsigned int*)adr = 0;                
        }
        
        //if (errors_count > 0)
        //        Send_Test_Report(CODE_MEMORY_TEST, 0x81, errors_count);
        //else
        //        Send_Test_Report(CODE_MEMORY_TEST, 0x00, 0x00000000);               

        cerr = ((CMN_REG_t*)CMN_REG)->EDAC_INT_MEM_CERR_CNT;
        ferr = ((CMN_REG_t*)CMN_REG)->EDAC_INT_MEM_FERR_CNT;
        if (cerr || ferr)
        {
                PRINT("                       CERR = 0x");
                PRINT_INT_HEX(cerr); 
                PRINT("   FERR = 0x");
                PRINT_INT_HEX(ferr);
                PRINT("\n");
        }
        
}


volatile unsigned int iterations;

void Memory_Test()
{
        iterations = 1;
  
        Test_result++;
        PRINT("       Memory test - start\n"); 
#if (TEMP < END_ADR)
        PRINT("         (Reduced version)\n"); 
#endif
        ErrorsDetected = 0;
        
        while (iterations)
        {          
                PRINT("               chess test\n"); 
                Chess_Test();
          
                PRINT("               counter test\n"); 
                Counter_Test();
                
                PRINT("               running 0 and running 1 test\n"); 
                Run01_Test();
                
                PRINT("               march test\n"); 
                Marsch_Test();               
    
                PRINT("               galloping test\n");
                Galloping_Test();
                
                if (ErrorsDetected)
                {
                        break;
                }
                
                iterations--;         
        }   
    
        if (!ErrorsDetected)
        {
          PRINT("                Test result - OK\n"); 
          Test_result--;
        }
        else
        {
          PRINT("                Errors total count = ");
          PRINT_INT(ErrorsDetected);
          PRINT("\n                Test result - FAILED\n"); 
        }
}
