/*!\file
 * \brief         SpaceWire.
 *
 * Copyright(C)   , 2017
 * Copyright(C)     
 *   .
 *
 *  ,    ,    
 *  ,       
 *  " ".     " " 
 *  - , " "    , 
 *   ,      , 
 *         . 
 *  " "       
 *  ,       ,  , 
 *      ,  - . 
 *  " "         
 *   .  " "     
 *   ,        
 *     .
 *
 *   , ,      
 *       " "  
 *       ,  , 
 *         
 * " ".         .
 */


#include "spacewire_test.h"


const uint32_t sys_freq = 80000000;                                             //    
const uint32_t spw_freq = 100000000;                                            //    Spacewire 

char dbg_buffer[256];                                                           //   

#define FAST_SPW_TEST                                                           //  
#define FULL_LOG                                                                //  

#ifdef FAST_SPW_TEST

#define SPW_TIMEOUT_INIT 8000000                                                  // 
#define SPW_TIMEOUT_TX 5000000                                                    //  TX
#define SPW_TIMEOUT_RX 5000000                                                    //  RX

unsigned int spw1_intr_flag;                                                    //  SPW1
unsigned int spw2_intr_flag;                                                    //  SPW2

/**
 * @brief    Spacewire 1 
 */
void Spacewire1_IRQ (void)
{
    spw1_intr_flag = 1;                                                         //  SPW1 = 1
}

/**
 * @brief    Spacewire 2 
 */
void Spacewire2_IRQ (void)
{
    spw2_intr_flag = 1;                                                         //  SPW2 = 1
}

/**
 * @brief   
 */
void SetDefaults(SPW_T* pSPW, unsigned char spw_number)
{     
    if (pSPW->STATUS & SW_STATUS_DISCONNECT_ERR)        
    {
        PRINT("Spacewire %i: Got disconnect error", spw_number);                //  
    }
        
    if (pSPW->STATUS & SW_STATUS_PARITY_ERR)
    {
        PRINT("Spacewire %i: Got parity error", spw_number);                    //   
    }
        
    if (pSPW->STATUS & SW_STATUS_ESCAPE_ERR)
    {
        PRINT("Spacewire %i: Got escape error", spw_number);                    //  escape        
    }
        
    if (pSPW->STATUS & SW_STATUS_CREDIT_ERR)
    {
        PRINT("Spacewire %i: Got credit error", spw_number);                    //  credit          
    }
        
    if (pSPW->STATUS & SW_STATUS_AHB_ERR)
    {
        PRINT("Spacewire %i: AHB error occured", spw_number);                   //   AHB                    
    }
        
    pSPW->CONTROL = SW_CONTROL_RESET_AMBA |                                     // 
                    SW_CONTROL_RESET_DMA |                                      // DMA 
                    SW_CONTROL_CANCEL_TX_DMA ;                                  // 1   TXDMA  RXDMA   
        
    pSPW->TXSCALER = 9;                                                         //    
    pSPW->RXDESCPTR = 0;                                                        //      
    pSPW->TXDESCPTR = 0;                                                        //       
}

/**
 * @brief   
 */
void ResetSPW()
{
    SetDefaults(SPW1, 1);                                                       //  
    SetDefaults(SPW2, 2);                                                       //  
}

#define PACKET_SIZE_WORDS  256                                                  //  
#pragma data_alignment = 0x200
#pragma location=".noinit"
SPW_DMA_DSC_TBL_T SPWA_DSC_TBL;
#pragma data_alignment = 0x200
#pragma location=".noinit"
SPW_DMA_DSC_TBL_T SPWB_DSC_TBL;
#pragma location=".noinit"
unsigned int txdataA [PACKET_SIZE_WORDS];                                       //   tx
#pragma location=".noinit"
unsigned int rxdataB [PACKET_SIZE_WORDS];                                       //   rx

unsigned char cur_dscr = 0;

/**
 * @brief  Spacewire 
 */
void InitSPWDscr(SPW_T* pSPW, SPW_DMA_DSC_TBL_T* pDSC_TBL, uint32_t* dscr_buf_tx, uint32_t* dscr_buf_rx)
{
    pDSC_TBL->TXDsc[cur_dscr].HEADER = (1<<16) |                                //EN 
                                     (PACKET_SIZE_WORDS*4);                     //   
    pDSC_TBL->TXDsc[cur_dscr].DATA_PTR = dscr_buf_tx;          
    pDSC_TBL->TXDsc[(cur_dscr+1)%SPW_DSC_COUNT].HEADER = 0;
    pDSC_TBL->TXDsc[(cur_dscr+1)%SPW_DSC_COUNT].DATA_PTR = 0;
    pSPW->TXDESCPTR = (uint32_t)pDSC_TBL->TXDsc + (cur_dscr << 3);
        
    pDSC_TBL->RXDsc[cur_dscr].HEADER = (1<<16) |                                //EN 
                                     (1<<18) |                                  
                                     (PACKET_SIZE_WORDS*4);                     //   
    pDSC_TBL->RXDsc[cur_dscr].DATA_PTR = dscr_buf_rx;          
    pDSC_TBL->RXDsc[(cur_dscr+1)%SPW_DSC_COUNT].HEADER = 0;
    pDSC_TBL->RXDsc[(cur_dscr+1)%SPW_DSC_COUNT].DATA_PTR = 0;
    pSPW->RXDESCPTR = (uint32_t)pDSC_TBL->RXDsc + (cur_dscr << 3);
        
    cur_dscr++;
    cur_dscr %= SPW_DSC_COUNT;        
}

unsigned int* spwA_intr_flag;                                                   //   SPWA
unsigned int* spwB_intr_flag;                                                   //   SPWB
unsigned char odd_even = 0;

/**
 * @brief   
 */
unsigned int SpacewireTransmit(SPW_T* pSPWA, SPW_T* pSPWB, int clk_divisor)
{          
    if (pSPWA == (SPW_T*)SPW1)
    {
        spwA_intr_flag = &spw1_intr_flag;
        spwB_intr_flag = &spw2_intr_flag;
#ifdef FULL_LOG
        PRINTSTREAM("Transmit from spw1 to spw2");
#endif                
    }
    else
    {
        spwA_intr_flag = &spw2_intr_flag;
        spwB_intr_flag = &spw1_intr_flag;
#ifdef FULL_LOG
        PRINTSTREAM("Transmit from spw2 to spw1");
#endif
    }
#ifdef FULL_LOG
    PRINT(", speed = %i Mbit/s\n", spw_freq/1000000/(clk_divisor+1));
#endif  
    pSPWA->TXSCALER = SW_TXSCALER_DIVISOR(clk_divisor);                         //      A
    pSPWB->TXSCALER = SW_TXSCALER_DIVISOR(clk_divisor);                         //      B
        
    //
    pSPWA->CONTROL &= ~SW_CONTROL_LINK_START;
    pSPWA->CONTROL |= SW_CONTROL_LINK_START;                                    // Linkstart;  1   .
  
    pSPWB->CONTROL &= ~SW_CONTROL_LINK_START;
    pSPWB->CONTROL &= ~SW_CONTROL_AUTOSTART;
    pSPWB->CONTROL |= SW_CONTROL_AUTOSTART;                                     // Autostart;  1       .
    
    pSPWA->CONTROL &= ~SW_CONTROL_INT_EN_RX_DSCR_COMPLETE;
    pSPWA->CONTROL |= SW_CONTROL_INT_EN_RX_DSCR_COMPLETE;                       //     RX ,   IE=1	
	 
    pSPWB->CONTROL &=~SW_CONTROL_INT_EN_RX_DSCR_COMPLETE; 
    pSPWB->CONTROL |= SW_CONTROL_INT_EN_RX_DSCR_COMPLETE;                       //     RX   IE = 1
        

    unsigned int timeout; 
    // 
    timeout = 0;	
    while(!((((pSPWA->STATUS)&SW_STATUS_LINK_STATUS_ENABLED)==0x03)&&
            (((pSPWB->STATUS)&SW_STATUS_LINK_STATUS_ENABLED)==0x03)))           // C Link : 
	if (timeout++>SPW_TIMEOUT_INIT)
	{
            PRINT("ERROR: spacewire connection timeout\n");
            ResetSPW();
            return 1;
	}	     
        
    unsigned int value = 0;
    for (unsigned int i=0; i<PACKET_SIZE_WORDS; i++)
    {     
        txdataA[i] = (odd_even)?value:~value;
        if (i < 32)
            value = (value << 1) + 1;
        else if (i < 64)
            value = (value << 1);
        else 
            value += 0x01010101;
    }
    InitSPWDscr(pSPWA, &SPWA_DSC_TBL, &txdataA[0], 0);                          
    InitSPWDscr(pSPWB, &SPWB_DSC_TBL, 0, &rxdataB[0]);
        
    spw1_intr_flag = 0;
    spw2_intr_flag = 0;               
        
    // TX_A RX_B DMA
    pSPWA->CONTROL |= SW_CONTROL_RESTART_TX_DMA;                                //  TXDMA A
    pSPWB->CONTROL |= SW_CONTROL_RESTART_RX_DMA;                                //  RXDMA B
                
    // TX_A
    timeout = 0;	
    while(!(SPWA_DSC_TBL.TXDsc[0].HEADER&(1<<19)))            
         if (timeout++>SPW_TIMEOUT_TX)
	 {
             PRINT("ERROR: spacewire TX_A timeout\n");
             ResetSPW();
             return 2;                 
	 }       
        
    // RX_B
    timeout = 0;
    while(*spwB_intr_flag==0)
    {
        if (timeout++>SPW_TIMEOUT_RX)
        {
            PRINT("ERROR: spacewire RX_B timeout\n");
            ResetSPW();
            return 3;                 
        } 
    }
        
    //
    unsigned char data_error = 0;
    for (unsigned int i=0; i<PACKET_SIZE_WORDS; i++)
        if (rxdataB[i] != txdataA[i])
        {
            PRINT("ERROR: spacewire data not correct\n");
            PRINT("Word #%i Expected: %x Received: %x", i, txdataA[i], rxdataB[i]);                                                   
            if (data_error++ > 10)
                break;
        }
    if (data_error)
    {
         ResetSPW();
         return 4;
    }        
        
    return 0; //OK
}

/**
 * @brief  GPIO   Spacewire 
 */
void Gpio_Spacewire_Init()
{
  
    GPIO_G->ALTFUNCSET |= (uint32_t)(1<<4);                                     // SW1_SIN                          
    GPIO_G->ALTFUNCSET |= (uint32_t)(1<<6);                                     // SW1_SOUT
    
    GPIO_G->ALTFUNCSET |= (uint32_t)(1<<5);                                     // SW1_DIN
    GPIO_G->ALTFUNCSET |= (uint32_t)(1<<7);                                     // SW1_DOUT
    
    GPIO_G->ALTFUNCSET |= (uint32_t)(1<<8);                                     // SW2_SIN 
    GPIO_G->ALTFUNCSET |= (uint32_t)(1<<10);                                    // SW2_SOUT
    
    GPIO_G->ALTFUNCSET |= (uint32_t)(1<<9);                                     // SW2_DIN
    GPIO_G->ALTFUNCSET |= (uint32_t)(1<<11);                                    // SW2_DOUT
    
}

/**
 * @brief  Spacewire 
 */
void SpacewireInternal_Test()
{
    PRINT("Spacewire test - start\n");
    SPW_Init(SPW1);                                                             //   SPW1       
    SPW_Init(SPW2);                                                             //   SPW2     
    SPW_freq(1);                                                                //               
    NVIC_EnableIRQ(Spacewire1,16);                                              //   SPW1
    NVIC_EnableIRQ(Spacewire2,17);                                              //   SPW2
    Gpio_Spacewire_Init();                                                      //   gpio     SpaceWire
    unsigned int result = 0;                                                    // 
    for(int clk = 0; clk<16; clk+=2)
    {
        result += SpacewireTransmit(SPW1, SPW2, clk);                           //    SPW1  SPW2        
        result += SpacewireTransmit(SPW2, SPW1, clk);                           //    SPW2  SPW1 
    }
    odd_even = (1-odd_even);  
    if (!result)
    {
        PRINT("Test result - OK\n");
    }
    else
    {
        PRINT("Test result - FAILED\n");
    }        
}

#endif

