/*!\file
 * \brief    ,            .
 *
 * Copyright(C)   , 2017
 * Copyright(C)     
 *   .
 *
 *  ,    ,    
 *  ,       
 *  " ".     " " 
 *  - , " "    , 
 *   ,      , 
 *         . 
 *  " "       
 *  ,       ,  , 
 *      ,  - . 
 *  " "         
 *   .  " "     
 *   ,        
 *     .
 *
 *   , ,      
 *       " "  
 *       ,  , 
 *         
 * " ".         .
 */

#ifndef DMA_H_
#define DMA_H_

#include "types.h"

/**
 * @brief       DMA
 */
typedef struct
{
    __RO uint32_t STATUS;                                                       /**< : 0x0000   DMA */
    __WO uint32_t CFG;                                                          /**< : 0x0004   DMA */
    __RW uint32_t CTRL_BASE_PTR;                                                /**< : 0x0008        */
    __RO uint32_t ALT_CTRL_BASE_PTR;                                            /**< : 0x000C       */
    __RO uint32_t WAITONREQ_STATUS;                                             /**< : 0x0010         */
    __WO uint32_t CHNL_SW_REQUEST;                                              /**< : 0x0014        */
    __RW uint32_t CHNL_USEBURST_SET;                                            /**< : 0x0018       */
    __WO uint32_t CHNL_USEBURST_CLR;                                            /**< : 0x001C      */
    __RW uint32_t CHNL_REQ_MASK_SET;                                            /**< : 0x0020       */
    __WO uint32_t CHNL_REQ_MASK_CLR;                                            /**< : 0x0024        */
    __RW uint32_t CHNL_ENABLE_SET;                                              /**< : 0x0028     */
    __WO uint32_t CHNL_ENABLE_CLR;                                              /**< : 0x002C      */
    __RW uint32_t CHNL_PRI_ALT_SET;                                             /**< : 0x0030   /      */
    __WO uint32_t CHNL_PRI_ALT_CLR;                                             /**< : 0x0034   /      */
    __RW uint32_t CHNL_PRIORITY_SET;                                            /**< : 0x0038     */
    __WO uint32_t CHNL_PRIORITY_CLR;                                            /**< : 0x003C     */
    __RW uint32_t RESERVED_0[3];                                     
    __RW uint32_t ERR_CLR;                                                      /**< : 0x04C     (R/W) */
    __RW uint32_t RESERVED_1[875];
}DMA_T;

/**
 * @brief       
 */
#define	DMA_CCTRL_STOP                  ((uint32_t)(0 << 0))                    /**< . C     */
#define	DMA_CCTRL_BASIC                 ((uint32_t)(1 << 0))                    /**<   .         DMA,        */
#define	DMA_CCTRL_AUTOREQUEST           ((uint32_t)(2 << 0))                    /**<   -.              */
#define DMA_CCTRL_PINGPONG              ((uint32_t)(3 << 0))                    /**<   -.    DMA     */
#define DMA_CCTRL_MEMSG_PRIM            ((uint32_t)(4 << 0))                    /**<   scatter-gather       */
#define	DMA_CCTRL_MEMSG_ALT             ((uint32_t)(5 << 0))                    /**<   scatter-gather       */
#define DMA_CCTRL_PERIPHSG_PRIM         ((uint32_t)(6 << 0))                    /**<   scatter-gather       */
#define DMA_CCTRL_PERIPHSG_ALT          ((uint32_t)(7 << 0))                    /**<   scatter-gather       */
 
  
#define DMA_NEXT_USEBURST_EN            ((uint32_t)(1 << 3))                    /**<    chnl_useburst_set[C]   1 */
#define DMA_NEXT_USEBURST_DIS           ((uint32_t)(0 << 3))                    /**<     chnl_useburst_set[C] */
  
#define DMA_N_MINUS_1(n)                ((uint32_t)((n & 0x07FF) << 4))         /**<    DMA (n+1),     DMA */
#define DMA_R_POWER(n)                  ((uint32_t)((n & 0x000F) << 14))        /**<   DMA (2^n)      */
  
#define DMA_SRC_PROT_CACHE_EN           ((uint32_t)(1 << 18))                   /**<       */
#define DMA_SRC_PROT_CACHE_DIS          ((uint32_t)(0 << 18))                   /**<       */
  
#define DMA_SRC_PROT_BUFFER_EN          ((uint32_t)(1 << 19))                   /**<       */
#define DMA_SRC_PROT_BUFFER_DIS         ((uint32_t)(0 << 19))                   /**<       */
  
#define DMA_SRC_PROT_PRIVILEGE_EN       ((uint32_t)(1 << 20))                   /**<        */
#define DMA_SRC_PROT_PRIVILEGE_DIS      ((uint32_t)(0 << 20))                   /**<        */
  
#define DMA_DST_PROT_CACHE_EN           ((uint32_t)(1 << 21))                   /**<       */
#define DMA_DST_PROT_CACHE_DIS          ((uint32_t)(0 << 21))                   /**<       */
  
#define DMA_DST_PROT_BUFFER_EN          ((uint32_t)(1 << 22))                   /**<       */
#define DMA_DST_PROT_BUFFER_DIS         ((uint32_t)(0 << 22))                   /**<       */
  
#define DMA_DST_PROT_PRIVILEGE_EN       ((uint32_t)(1 << 23))                   /**<        */
#define DMA_DST_PROT_PRIVILEGE_DIS      ((uint32_t)(0 << 23))                   /**<        */

#define	DMA_SRC_DATA_SIZE_BYTE          ((uint32_t)(0 << 24))                   /**<    8  */ 
#define	DMA_SRC_DATA_SIZE_HALFWORD      ((uint32_t)(1 << 24))                   /**<    16  */ 
#define	DMA_SRC_DATA_SIZE_WORD          ((uint32_t)(2 << 24))                   /**<    32  */ 
 
#define	DMA_SRC_ADDR_INC_BYTE           ((uint32_t)(0 << 26))                   /**<     8  */ 
#define	DMA_SRC_ADDR_INC_HALFWORD       ((uint32_t)(1 << 26))                   /**<     16  */ 
#define	DMA_SRC_ADDR_INC_WORD           ((uint32_t)(2 << 26))                   /**<     32  */
#define	DMA_SRC_ADDR_INC_NULL           ((uint32_t)(3 << 26))                   /**<     */ 

#define	DMA_DST_DATA_SIZE_BYTE          ((uint32_t)(0 << 28))                   /**<    8  */ 
#define	DMA_DST_DATA_SIZE_HALFWORD      ((uint32_t)(1 << 28))                   /**<    16  */ 
#define	DMA_DST_DATA_SIZE_WORD          ((uint32_t)(2 << 28))                   /**<    32  */ 

#define	DMA_DST_ADDR_INC_BYTE           ((uint32_t)(0 << 30))                   /**<     8  */ 
#define	DMA_DST_ADDR_INC_HALFWORD       ((uint32_t)(1 << 30))                   /**<     16  */ 
#define	DMA_DST_ADDR_INC_WORD           ((uint32_t)(2 << 30))                   /**<     32  */
#define	DMA_DST_ADDR_INC_NULL           ((uint32_t)(3 << 30))                   /**<     */ 
   
/**
 * @brief   
 */
typedef struct 
{
    uint32_t src_data_end_ptr;                                                  /**<      */
    uint32_t dst_data_end_ptr;                                                  /**<       */
    volatile uint32_t ctrl;                                                     /**<   */
    uint32_t reserved;                                                          /**< */
}DMA_CHNL_CTRL_T;

/**
 * @brief        
 */
typedef struct 
{              
    DMA_CHNL_CTRL_T prim_chan[32];                                              /**<     */
    DMA_CHNL_CTRL_T alt_chan[32];                                               /**<     */
}DMA_CTRL_T;


#endif /* DMA_H_ */