/*!\file
 * \brief    ,          GPIO.
 *
 * Copyright(C)   , 2017
 * Copyright(C)     
 *   .
 *
 *  ,    ,    
 *  ,       
 *  " ".     " " 
 *  - , " "    , 
 *   ,      , 
 *         . 
 *  " "       
 *  ,       ,  , 
 *      ,  - . 
 *  " "         
 *   .  " "     
 *   ,        
 *     .
 *
 *   , ,      
 *       " "  
 *       ,  , 
 *         
 * " ".         .
 */

#ifndef GPIO_H_
#define GPIO_H_

#include "types.h"

/**
 * @brief      GPIO.
 */
typedef struct 
{
    __RW uint32_t  DATA;                                                        /**< : 0x0000        */ 
    __RW uint32_t  DATAOUT;                                                     /**< : 0x0004         */ 
    __RW uint32_t  RESERVED0[2];                                                /**< */
    __RW uint32_t  OUTENSET;                                                    /**< : 0x0010       */
    __RW uint32_t  OUTENCLR;                                                    /**< : 0x0014        */
    __RW uint32_t  ALTFUNCSET;                                                  /**< : 0x0018        */
    __RW uint32_t  ALTFUNCCLR;                                                  /**< : 0x001C       GPIO*/
    __RW uint32_t  INTENSET;                                                    /**< : 0x0020     */
    __RW uint32_t  INTENCLR;                                                    /**< : 0x0024      */
    __RW uint32_t  INTTYPESET;                                                  /**< : 0x0028     : */
    __RW uint32_t  INTTYPECLR;                                                  /**< : 0x002C      */
    __RW uint32_t  INTPOLSET;                                                   /**< : 0x0030       */ 
    __RW uint32_t  INTPOLCLR;                                                   /**< : 0x0034       */
    __RW uint32_t  INTSTATUSCLEAR;                                              /**< : 0x0038   /   */
    __RW uint32_t  RESERVED1[241];                                              ///< 
    __RW uint32_t  LB_MASKED[256];                                              /**< : 0x0400 - 0x07FC        8-  */
    __RW uint32_t  UB_MASKED[256];                                              /**< : 0x0800 - 0x0BFC        8-  */
}GPIO_T;

/**
 * @brief  -        .
 */
typedef enum GPIOPINT_MODE 
{
    GPIOPINT_FRONT_HIGHT    = 0x02,                                             /**< b'10':  1 -  , 0 -   */
    GPIOPINT_FRONT_LOW      = 0x00,                                             /**< b'00':  0 -  , 0 -   */
    GPIOPINT_RISING_EDGE    = 0x03,                                             /**< b'11':  1 -   , 1 -   */
    GPIOPINT_FALLING_EDGE   = 0x01,                                             /**< b'01':  0 -   , 1 -   */
}GPIOPINT_MODE_T;


/**
 * @brief    GPIO
 */

/**   GPIO */   
#define	GPIO_IntEnable(gl_gpio_t, gl_gpio_pin) (((GPIO_T*)gl_gpio_t)->INTENSET = gl_gpio_pin)

/**   GPIO */   
#define	GPIO_IntDisable(gl_gpio_t, gl_gpio_pin) (((GPIO_T*)gl_gpio_t)->INTENCLR = gl_gpio_pin)

/**   GPIO */   
#define	GPIO_IntClear(gl_gpio_t, gl_gpio_pin) (((GPIO_T*)gl_gpio_t)->INTSTATUSCLEAR = gl_gpio_pin)

/**
 * @brief    GPIO
 * @param pGPIO:      GPIO.
 * @param Pin_number:   (0..15).
 * @param Dir:     
 *          Dir   :
 *          @arg true:    
 *          @arg false:   
 * @retval None.
 */
void GPIO_WriteDirBit(GPIO_T *pGPIO, uint8_t Pin_number, bool Dir);

/**
 * @brief    GPIO
 * @param pGPIO:      GPIO.
 * @param Pin_number:   (0..15).
 * @param Dir:     
 *          Dir   :
 *          @arg true:    
 *          @arg false:   
 * @retval None.
 */
void GPIO_SetDir(GPIO_T *pGPIO, uint32_t bitVal, bool dir);
 /**
 * @brief   GPIO   .
 * @note    :  B: 0x0007,  D: 0x0800,  E: 0x0403, 
 *          0x0000.
 * @param pGPIO:      GPIO.
 * @param Pin_number:   (0..15).
 * @param AltSet:  . 
 *          AltSet   :
 *          @arg true:   , 
 *          @arg false:  GPIO
 * @retval None.
 */
void GPIO_SetAltFunc(GPIO_T *pGPIO, uint16_t Pin_number, bool AltSet);

 /**
 * @brief     GPIO.
 * @param pGPIO:      GPIO.
 * @param Pin_number:   (0..15).
 * @param val:    . 
 * @retval None.
 */
void GPIO_WritePort(GPIO_T *pGPIO, uint16_t Pin_number, uint16_t val);

 /**
 * @brief      GPIO.
 * @param pGPIO:      GPIO.
 * @param pin:   (0..15).
 * @param val:  . 
 *          val   :
 *          @arg true:  , 
 *          @arg false:  
 * @retval None.
 */
void GPIO_WritePortBit(GPIO_T *pGPIO, uint8_t pin, bool val);

/**
 * @brief      	
 * @param pGPIO:      GPIO.
 * @retval   
 */
uint32_t GPIOOUT_ReadPort(GPIO_T *pGPIO);

/**
 * @brief      	
 * @param pGPIO:      GPIO.
 * @retval   
 */
uint32_t GPIOIN_ReadPort(GPIO_T *pGPIO);

/**
 * @brief      
 * @param pGPIO:      GPIO.
 * @param pin:   (0..15).
 * @retval   :
 * @note  true - , false - 
 */
bool GPIOOUT_ReadPortBit(GPIO_T *pGPIO, uint8_t pin);

/**
 * @brief       
 * @param pGPIO:      GPIO.
 * @param pin:   (0..15).
 * @retval   :
 * @note  true - , false - 
 */
bool GPIOIN_ReadPortBit(GPIO_T *pGPIO, uint8_t pin);

/**
 * @brief    GPIO
 * @param pGPIO:      GPIO.
 * @param pin:   (0..15).
 * @retval   :
 * @note  true -  , false -  
 */
bool GPIO_ReadDirBit(GPIO_T *pGPIO, uint8_t pin);

/**
 * @brief    GPIO
 * @param pGPIO:      GPIO.
 * @retval   
 */
uint32_t GPIO_GetDir(GPIO_T *pGPIO);

/**
 * @brief   , GPIO     
 * @note    :  B: 0x0007,  D: 0x0800,  E: 0x0403, 
 *          0x0000.
 * @param pGPIO:      GPIO.
 * @retval  : GPIO   . 
 */
uint32_t GPIO_GetAltFunc(GPIO_T *pGPIO);

/**
 * @brief   .
 * @param pGPIO:      GPIO.
 * @param Pin_number:   (0..15).
 * @param IntMode:   .
 *          GPIOPINT_MODE_T   :
 *          @arg GPIOPINT_FRONT_HIGHT:   ,   
 *          @arg GPIOPINT_FRONT_LOW:   ,   
 *          @arg GPIOPINT_RISING_EDGE:   ,   
 *          @arg GPIOPINT_FALLING_EDGE:   ,   
 * @retval None.
 */
void GPIO_IntConfig(GPIO_T *pGPIO, uint32_t Pin_number, GPIOPINT_MODE_T IntMode);

/**
 * @brief       
 * @param pGPIO:      GPIO.
 * @retval   
 */
uint32_t GPIO_IntGetStatus(GPIO_T *pGPIO);

/**
 * @brief      
 * @param pGPIO:      GPIO.
 * @param pin:   (0..15).
 * @retval   
 */ 
GPIOPINT_MODE_T GPIO_GetIntConfigPin(GPIO_T *pGPIO, uint8_t pin);        



#endif /* GPIO_H */
