/*!\file
 * \brief           GPIO.
 *
 * Copyright(C)   , 2017
 * Copyright(C)     
 *   .
 *
 *  ,    ,    
 *  ,       
 *  " ".     " " 
 *  - , " "    , 
 *   ,      , 
 *         . 
 *  " "       
 *  ,       ,  , 
 *      ,  - . 
 *  " "         
 *   .  " "     
 *   ,        
 *     .
 *
 *   , ,      
 *       " "  
 *       ,  , 
 *         
 * " ".         .
 */

#include "gpio.h"


/**
 * @brief    GPIO
 * @param pGPIO:      GPIO.
 * @param Pin_number:   (0..15).
 * @param Dir:     
 *          Dir   :
 *          @arg true:    
 *          @arg false:   
 * @retval None.
 */
void GPIO_WriteDirBit(GPIO_T *pGPIO, uint8_t Pin_number, bool Dir)
{
    if (Dir) 
    {
        pGPIO->OUTENSET = (1UL << Pin_number);
    }
    else 
    {
	pGPIO->OUTENCLR = (1UL << Pin_number);
    }
}

/**
 * @brief    GPIO
 * @param pGPIO:      GPIO.
 * @param Pin_number:   (0..15).
 * @param Dir:     
 *          Dir   :
 *          @arg true:    
 *          @arg false:   
 * @retval None.
 */
void GPIO_SetDir(GPIO_T *pGPIO, uint32_t bitVal, bool dir)
{
    if (dir) 
    {
        pGPIO->OUTENSET |= bitVal;
    }
    else 
    {
        pGPIO->OUTENCLR |= bitVal;
    }
}

 /**
 * @brief   GPIO   .
 * @note    :  B: 0x0007,  D: 0x0800,  E: 0x0403, 
 *          0x0000.
 * @param pGPIO:      GPIO.
 * @param Pin_number:   (0..15).
 * @param AltSet:  . 
 *          AltSet   :
 *          @arg true:   , 
 *          @arg false:  GPIO
 * @retval None.
 */
void GPIO_SetAltFunc(GPIO_T *pGPIO, uint16_t Pin_number, bool AltSet)
{
    if(AltSet)
    {
        pGPIO->ALTFUNCSET |= (1UL << Pin_number);
    } 
    else 
    {
        pGPIO->ALTFUNCCLR |= (1UL << Pin_number);
    }
}


 /**
 * @brief     GPIO.
 * @param pGPIO:      GPIO.
 * @param Pin_number:   (0..15).
 * @param val:    . 
 * @retval None.
 */
void GPIO_WritePort(GPIO_T *pGPIO, uint16_t Pin_number, uint16_t val)
{
    if(Pin_number & 0x00FF)
    {
        pGPIO->LB_MASKED[Pin_number & 0x00FF] = val;
    }
    if((Pin_number & 0xFF00)>>8)
    {
        pGPIO->UB_MASKED[(Pin_number & 0xFF00)>>8] = val;
    }
}

 /**
 * @brief      GPIO.
 * @param pGPIO:      GPIO.
 * @param pin:   (0..15).
 * @param val:  . 
 *          val   :
 *          @arg true:  , 
 *          @arg false:  
 * @retval None.
 */
void GPIO_WritePortBit(GPIO_T *pGPIO, uint8_t pin, bool val)
{
    if(pin<8)
    {
        pGPIO->LB_MASKED[1 << pin] = val << pin;
    }
    else
    {
        pGPIO->UB_MASKED[(1 << pin)>>8] = (val << pin);
    }
}

/**
 * @brief      	
 * @param pGPIO:      GPIO.
 * @retval   
 */
uint32_t GPIOOUT_ReadPort(GPIO_T *pGPIO)
{
    //return pGPIO->UB_MASKED[255]|pGPIO->LB_MASKED[255];
    return pGPIO->DATAOUT;
}

/**
 * @brief      	
 * @param pGPIO:      GPIO.
 * @retval   
 */
uint32_t GPIOIN_ReadPort(GPIO_T *pGPIO)
{
    return pGPIO->DATA;
}

/**
 * @brief      
 * @param pGPIO:      GPIO.
 * @param pin:   (0..15).
 * @retval   :
 * @note  true - , false - 
 */
bool GPIOOUT_ReadPortBit(GPIO_T *pGPIO, uint8_t pin)
{
    return (bool)(((pGPIO->DATAOUT)&(1<<pin))>>pin);
}

/**
 * @brief       
 * @param pGPIO:      GPIO.
 * @param pin:   (0..15).
 * @retval   :
 * @note  true - , false - 
 */
bool GPIOIN_ReadPortBit(GPIO_T *pGPIO, uint8_t pin)
{
   return (bool)(((pGPIO->DATA)&(1<<pin))>>pin);
}

/**
 * @brief    GPIO
 * @param pGPIO:      GPIO.
 * @param pin:   (0..15).
 * @retval   :
 * @note  true -  , false -  
 */
bool GPIO_ReadDirBit(GPIO_T *pGPIO, uint8_t pin)
{
    return (bool) (((pGPIO->OUTENSET) >> pin) & 1);
}

/**
 * @brief    GPIO
 * @param pGPIO:      GPIO.
 * @retval   
 */
uint32_t GPIO_GetDir(GPIO_T *pGPIO)
{
    return pGPIO->OUTENSET;
}

/**
 * @brief   , GPIO     
 * @note    :  B: 0x0007,  D: 0x0800,  E: 0x0403, 
 *          0x0000.
 * @param pGPIO:      GPIO.
 * @retval  : GPIO   . 
 */
uint32_t GPIO_GetAltFunc(GPIO_T *pGPIO)
{
    return pGPIO->ALTFUNCSET;
}



/**
 * @brief   .
 * @param pGPIO:      GPIO.
 * @param Pin_number:   (0..15).
 * @param IntMode:   .
 *          GPIOPINT_MODE_T   :
 *          @arg GPIOPINT_FRONT_HIGHT:   ,   
 *          @arg GPIOPINT_FRONT_LOW:   ,   
 *          @arg GPIOPINT_RISING_EDGE:   ,   
 *          @arg GPIOPINT_FALLING_EDGE:   ,   
 * @retval None.
 */
void GPIO_IntConfig(GPIO_T *pGPIO, uint32_t Pin_number, GPIOPINT_MODE_T IntMode)
{
    switch(IntMode)
    {
        case GPIOPINT_FRONT_HIGHT:
            pGPIO->INTTYPESET = Pin_number;
            pGPIO->INTPOLSET =  Pin_number;
            break;
        case GPIOPINT_FRONT_LOW:
            pGPIO->INTTYPESET = Pin_number;
            pGPIO->INTPOLCLR =  Pin_number;
            break;
        case GPIOPINT_RISING_EDGE:
            pGPIO->INTTYPECLR = Pin_number;
            pGPIO->INTPOLSET =  Pin_number;
            break;
        case GPIOPINT_FALLING_EDGE:
            pGPIO->INTTYPECLR = Pin_number;
            pGPIO->INTPOLCLR =  Pin_number;
            break;
        default:
            break;
  }
}

/**
 * @brief       
 * @param pGPIO:      GPIO.
 * @retval   
 */
uint32_t GPIO_IntGetStatus(GPIO_T *pGPIO)
{
    return pGPIO->INTSTATUSCLEAR;
}

/**
 * @brief      
 * @param pGPIO:      GPIO.
 * @param pin:   (0..15).
 * @retval   
 */ 
GPIOPINT_MODE_T GPIO_GetIntConfigPin(GPIO_T *pGPIO, uint8_t pin)
{
    return (GPIOPINT_MODE_T)(((((pGPIO->INTTYPESET)&(1<<pin))>>pin)<<1) | ((pGPIO->INTPOLSET&(1<<pin))>>pin));
}        


